# Generated by Django 5.1.3 on 2024-11-28 15:00

from django.db import migrations


class Migration(migrations.Migration):
    """Fixes `badly formed hexadecimal UUID string` exception.

    ... raised when running `model_cls.objects.all().delete()` on some
    report models (with existing int [autoincrement] values in their id
    fields), that Django is unable to handle.
    """

    dependencies = [
        ("intecomm_reports", "0028_auto_20241031_0602"),
    ]

    operations = [
        migrations.RunSQL("delete from intecomm_reports_missingvldrawdates;"),
        migrations.RunSQL("delete from intecomm_reports_vlsummary6m;"),
        migrations.RunSQL("delete from intecomm_reports_vlsummary9m;"),
    ]



import pandas as pd
from django.contrib.sites.models import Site
from django_pandas.io import read_frame
from edc_model import duration_to_date

from intecomm_screening.models import PatientLog, SubjectScreening
from intecomm_subject.models import SubjectVisit


def duration_to_date_by_row(row: pd.Series, col: str = None):
    return duration_to_date(duration_text=row[col], reference_date=row["report_datetime"])


def get_screening_df(df: pd.DataFrame | None = None) -> pd.DataFrame:
    df = pd.DataFrame() if not hasattr(df, "empty") else df
    if df.empty:
        exclude = [
            "initials",
            "legal_name",
            "familiar_name",
            "hospital_identifier",
            "hostname_created",
            "hostname_modified",
            "device_created",
            "device_modified",
            "locale_created",
            "locale_modified",
            "subject_identifier_aka",
            "slug",
            "subject_identifier_as_pk",
        ]
        fldnames = [
            fld.name for fld in SubjectScreening._meta.get_fields() if fld.name not in exclude
        ]
        qs_screening = SubjectScreening.objects.values(*fldnames).all()
        df = read_frame(qs_screening)

    # df["gender"] = df["gender"].apply(lambda x: "F" if x == "Female" else x)
    # df["gender"] = df["gender"].apply(lambda x: "M" if x == "Male" else x)

    sites = {obj.name.title(): obj.id for obj in Site.objects.all()}
    df["site_id"] = df["site"].map(sites)

    # replace
    df = df.replace({"Not Applicable: e.g. male or post-menopausal": "N/A"})
    df = df.replace({"Not applicable": "N/A"})
    df = df.replace({True: 1})
    df = df.replace({False: 0})

    # convert all to float
    cols = [
        "age_in_years",
        "dia_blood_pressure_avg",
        "dia_blood_pressure_one",
        "dia_blood_pressure_two",
        "sys_blood_pressure_avg",
        "sys_blood_pressure_one",
        "sys_blood_pressure_two",
    ]
    df[cols] = df[cols].apply(pd.to_numeric)

    # convert to datetime
    cols = ["report_datetime", "eligibility_datetime", "real_eligibility_datetime", "created"]
    df[cols] = df[cols].apply(pd.to_datetime)

    # calc duration fields
    cols = ["hiv_dx_ago", "dm_dx_ago", "htn_dx_ago"]
    for col in cols:
        new_col = col.replace("ago", "date")
        df[new_col] = pd.NaT
        df[new_col] = df[df[col].notna()].apply(duration_to_date_by_row, axis=1, col=col)
    df["in_care_duration_as_date"] = pd.NaT
    df["in_care_duration_as_date"] = df[df["in_care_duration"].notna()].apply(
        duration_to_date_by_row, axis=1, col="in_care_duration"
    )

    # grouped
    qs_patientlog = PatientLog.objects.values(
        "screening_identifier",
        "group_identifier",
        # "conditions",
        "stable",
        "willing_to_screen",
        "screening_refusal_reason",
        "screening_refusal_reason_other",
    ).all()
    df_patientlog = read_frame(qs_patientlog)
    df = df.merge(df_patientlog, on="screening_identifier", how="left", suffixes=("", "_y"))

    # attended baseline visit
    qs_subjectvisit = SubjectVisit.objects.values(
        "appointment__subject_identifier", "report_datetime"
    ).filter(visit_code="1000", visit_code_sequence=0)
    df_subjectvisit = read_frame(qs_subjectvisit)
    df_subjectvisit = df_subjectvisit.rename(
        columns={"appointment__subject_identifier": "subject_identifier"}
    )
    df = df.merge(df_subjectvisit, on="subject_identifier", how="left", suffixes=("", "_y"))

    # convert datetimes to date
    date_cols = list(df.select_dtypes(include=["datetime64", "datetime64[ns, UTC]"]).columns)
    for col in date_cols:
        df[col] = df[col].dt.normalize()
    return df



from .appointment_location_qa import AppointmentLocationQa
from .dbviews import (  # OnStudyMissingLabValues,; OnStudyMissingValues,
    Eos,
    SubjectsTransferred,
    Vl,
)
from .diagnoses import Diagnoses
from .missing_vl_draw_dates import MissingVlDrawDates
from .vl_summary import VlSummary6m, VlSummary9m



# Generated by Django 5.1 on 2024-10-04 14:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0021_onstudymissinglabvalues_onstudymissingvalues_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="ApptLocationQa",
            new_name="AppointmentLocationQa",
        ),
        migrations.RenameIndex(
            model_name="appointmentlocationqa",
            new_name="intecomm_re_modifie_c2590e_idx",
            old_name="intecomm_re_modifie_4638df_idx",
        ),
        migrations.RenameIndex(
            model_name="appointmentlocationqa",
            new_name="intecomm_re_user_mo_50d666_idx",
            old_name="intecomm_re_user_mo_927bd2_idx",
        ),
    ]



import pandas as pd

from ..constants import DM_HTN, DM_ONLY, HIV_DM, HIV_DM_HTN, HIV_HTN, HIV_ONLY, HTN_ONLY


def cond_value_counts(source_df: pd.DataFrame, hiv: int, htn: int, dm: int) -> pd.Series:
    return source_df[
        (source_df["hiv"] == hiv) & (source_df["htn"] == htn) & (source_df["dm"] == dm)
    ]["country"].value_counts()


def get_conditions_by_country(
    source_df: pd.DataFrame, summarize: bool | None = None
) -> pd.DataFrame:
    """Given a dataframe calculates a freq table for each condition
    by country.

    For example:

        # with patient log
        patientlog_df = get_patientlog_df()
        df = get_conditions_by_country(patientlog_df, summarize=True)

        # with diagnoses table
        dx_df = get_diagnoses_df()
        df = get_conditions_by_country(dx_df, summarize=True)

    """
    df = pd.DataFrame()
    for index, data in enumerate(
        [
            (1, 0, 0, HIV_ONLY),
            (0, 1, 0, HTN_ONLY),
            (0, 0, 1, DM_ONLY),
            (1, 0, 1, HIV_DM),
            (1, 1, 0, HIV_HTN),
            (1, 1, 1, HIV_DM_HTN),
            (0, 1, 1, DM_HTN),
        ]
    ):
        hiv, htn, dm, label = data
        df1 = cond_value_counts(source_df, hiv, htn, dm).to_frame().reset_index()
        df1["label"] = label
        df = pd.concat([df, df1])
    df = df.reset_index(drop=True)
    if summarize:
        df = df.pivot(index="label", columns=["country"], values=["count"])
        df.columns = [col[1] for col in df.columns]
        df = df.reset_index()
        df["total"] = df["uganda"] + df["tanzania"]
        df.loc[len(df)] = {
            "label": "total",
            "uganda": df["uganda"].sum(),
            "tanzania": df["tanzania"].sum(),
            "total": df["total"].sum(),
        }
        # column props
        df["ug_cprop"] = df["uganda"] / df.loc[7]["uganda"]
        df["tz_cprop"] = df["tanzania"] / df.loc[7]["tanzania"]
        df["t_cprop"] = (df["uganda"] + df["tanzania"]) / df.loc[7]["total"]
        # row prop
        df["ug_rprop"] = df["uganda"] / df["total"]
        df["tz_rprop"] = df["tanzania"] / df["total"]
        df["t_rprop"] = df["ug_rprop"] + df["tz_rprop"]
    return df



import pandas as pd
from django_pandas.io import read_frame
from edc_appointment.models import Appointment, AppointmentType


def get_appointment_df() -> pd.DataFrame:
    df_appt = read_frame(Appointment.objects.all(), verbose=False)
    df1 = read_frame(AppointmentType.objects.values("id", "name").all())
    df1 = df1.rename(columns={"id": "appt_type"})
    df_appt = df_appt.merge(df1, on="appt_type", how="left")
    df_appt = df_appt.drop(columns=["appt_type"])
    df_appt = df_appt.rename(columns={"name": "appt_type"})
    return df_appt



from django.contrib import admin
from django.db.models import QuerySet
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext as _
from edc_appointment.models import Appointment
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_pdutils.utils import refresh_model_from_dataframe
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter

from intecomm_analytics.dataframes import get_appointment_location_qa_df

from ..admin_site import intecomm_reports_admin
from ..models import AppointmentLocationQa


@admin.register(AppointmentLocationQa, site=intecomm_reports_admin)
class AppointmentLocationQaAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    qa_report_list_display_insert_pos = 2
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "subject_identifier",
        "visit_code",
        "visit_code_sequence",
    ]

    list_filter = [
        ScheduleStatusListFilter,
        "visit_code",
        "visit_code_sequence",
    ]

    readonly_fields = (
        "subject_identifier",
        "visit_code",
        "visit_code_sequence",
        "site",
    )

    search_fields = ["subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        df = get_appointment_location_qa_df()
        refresh_model_from_dataframe(df, model_cls=AppointmentLocationQa)
        qs = super().get_queryset(request)
        return qs

    def dashboard(self, obj=None, label=None) -> str:
        appointment = Appointment.objects.get(
            subject_identifier=obj.subject_identifier,
            visit_code=obj.visit_code,
            visit_code_sequence=obj.visit_code_sequence,
        )

        dashboard_url = reverse(
            f"edc_appointment_admin:"
            f"{Appointment._meta.label_lower.replace('.', '_')}_change",
            args=(appointment.id,),
        )
        dashboard_url = (
            f"{dashboard_url}?next={self.admin_site.name}:"
            f"{self.model._meta.label_lower.replace('.', '_')}_changelist"
        )
        title = _(f"Change {Appointment._meta.verbose_name}")
        label = _(f"Change {Appointment._meta.verbose_name}")

        context = dict(title=title, url=dashboard_url, label=label)
        return render_to_string("dashboard_button.html", context=context)



# Generated by Django 5.1.2 on 2024-10-31 02:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0026_vlsummary6m_device_created_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="OnStudyMissingLabValues",
        ),
        migrations.DeleteModel(
            name="OnStudyMissingValues",
        ),
    ]



# Generated by Django 5.1 on 2024-08-29 12:58

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0037_remove_actionitem_reference_model_and_more"),
        ("intecomm_subject", "0188_delete_diagnoses"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="BloodResultsLipid",
            new_name="BloodResultsLipids",
        ),
        migrations.RenameModel(
            old_name="HistoricalBloodResultsLipid",
            new_name="HistoricalBloodResultsLipids",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="intecomm_su_subject_28e6f3_idx",
            old_name="intecomm_su_subject_0dd0d1_idx",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="intecomm_su_subject_081f29_idx",
            old_name="intecomm_su_subject_d3a470_idx",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="intecomm_su_modifie_40ed7c_idx",
            old_name="intecomm_su_modifie_2deb12_idx",
        ),
        migrations.RenameIndex(
            model_name="bloodresultslipids",
            new_name="intecomm_su_user_mo_8696f9_idx",
            old_name="intecomm_su_user_mo_4870e2_idx",
        ),
    ]



from .get_conditions_by_country import get_conditions_by_country
from .get_conditions_by_gender import get_conditions_by_gender
from .get_consent_df import get_consent_df
from .get_diagnoses_df import get_diagnoses_df
from .get_eos_df import get_eos_df
from .get_patientlog_df import get_patientlog_df
from .get_screening_df import duration_to_date_by_row, get_screening_df
from .qa import get_appointment_location_qa_df



# import pandas as pd
# from edc_analytics.custom_tables import (
#     AgeTable,
#     ArtTable,
#     BmiTable,
#     BpTable,
#     FastingTable,
#     FbgOgttTable,
#     FbgTable,
#     GenderTable,
#     HbA1cTable,
#     OgttTable,
#     WaistCircumferenceTable,
# )
# from edc_analytics.data import Data
#
# from .dataframes import get_baseline_df
#
# # from .tables import EligibleTable, HasDmTable
#
#
# def get_tables() -> dict[str, Data]:
#     """
#     results = get_table()
#     results.get("fbg").to_csv(folder="/Users/erikvw/Documents/")
#     """
#     df_screening = get_screening_df()
#     df_has_dm = df_screening[(df_screening["has_dm"] == "Yes")]
#     df_no_dm = df_screening[(df_screening["has_dm"] == "No")]
#     df_fbg = get_glucose_tested_only_df(df_screening)
#
#     df_not_tested = df_no_dm.copy()
#     df_not_eligible_p1p2 = df_not_tested[
#         ~(
#             (df_not_tested["eligible_part_one"] == "Yes")
#             & (df_not_tested["eligible_part_two"] == "Yes")
#         )
#     ]
#     df_not_tested.drop(df_not_eligible_p1p2.index, inplace=True)
#     df_not_tested.drop(df_fbg.index, inplace=True)
#
#     df_stats = pd.DataFrame(
#         {
#             "df": ["df_screening", "df_has_dm", "df_no_dm", "df_not_tested", "df_fbg"],
#             "count": [
#                 len(df_screening),
#                 len(df_has_dm),
#                 len(df_no_dm),
#                 len(df_not_tested),
#                 len(df_fbg),
#             ],
#         }
#     )
#
#     results = {"stats": df_stats}
#     for key, df in {
#         "screening": df_screening,
#         "has_dm": df_has_dm,
#         "no_dm": df_no_dm,
#         "not_tested": df_not_tested,
#         "fbg": df_fbg,
#     }.items():
#         dfs = []
#         for tbl_cls in [
#             GenderTable,
#             AgeTable,
#             WaistCircumferenceTable,
#             BmiTable,
#             BpTable,
#             ArtTable,
#             FastingTable,
#             FbgTable,
#             OgttTable,
#             FbgOgttTable,
#             HbA1cTable,
#             EligibleP12Table,
#             HasDmTable,
#         ]:
#             tbl = tbl_cls(main_df=df)
#             dfs.append(tbl.table_df)
#         results.update({key: Data(key, pd.concat(list(dfs)), df, "meta3")})
#     return results



# Generated by Django 5.1.2 on 2024-10-31 01:54

import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0024_alter_missingvldrawdates_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="vlsummary6m",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow),
        ),
        migrations.AddField(
            model_name="vlsummary9m",
            name="modified",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow),
        ),
    ]



import pandas as pd
from edc_model import duration_to_date
from edc_pdutils.dataframes import get_subject_consent

from intecomm_consent.models import SubjectConsent


def duration_to_date_by_row(row: pd.Series, col: str = None):
    return duration_to_date(duration_text=row[col], reference_date=row["report_datetime"])


def get_consent_df(df: pd.DataFrame | None = None) -> pd.DataFrame:
    df = get_subject_consent(model_cls=SubjectConsent, extra_columns=["group_identifier"])
    return df



# Generated by Django 5.0.8 on 2024-08-27 00:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_reports",
            "0018_alter_eos_options_alter_subjectstransferred_options_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="diagnoses",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Current Conditions",
                "verbose_name_plural": "Current Conditions",
            },
        ),
        migrations.AlterModelOptions(
            name="missingvldrawdates",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Viral load: Missing draw date",
                "verbose_name_plural": "Viral load: Missing draw dates",
            },
        ),
        migrations.AlterModelOptions(
            name="vlsummary6m",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Viral load summary (endline >= 6m)",
                "verbose_name_plural": "Viral load summary (endline >= 6m)",
            },
        ),
        migrations.AlterModelOptions(
            name="vlsummary9m",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Viral load summary (endline >= 9m)",
                "verbose_name_plural": "Viral load summary (endline >= 9m)",
            },
        ),
    ]



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    import pandas as pd

from intecomm_analytics.dataframes.get_appointment_df import get_appointment_df


def get_appointment_location_qa_df() -> pd.DataFrame:
    """Returns a dataframe of 1120 visits unexpectedly
    flagged as in the community.

    All 1120 appointment are conducted in the facility regardless
    of arm.
    """
    df_appt = get_appointment_df()
    cond_1120 = df_appt.visit_code == "1120"
    cond_status = (
        (df_appt.appt_status == "done")
        | (df_appt.appt_status == "incomplete")
        | (df_appt.appt_status == "in_progress")
    )
    df = df_appt[
        cond_1120
        & cond_status
        & (df_appt.schedule_name == "comm_schedule")
        & (df_appt.appt_type == "community")
    ][["subject_identifier", "site", "visit_code", "visit_code_sequence"]]
    df = df.rename(columns={"site": "site_id"})
    return df



import pandas as pd
from django.contrib.sites.models import Site
from django_pandas.io import read_frame
from edc_constants.constants import OTHER
from edc_sites.site import sites

from intecomm_consent.models import SubjectConsent
from intecomm_lists.list_data import list_data
from intecomm_prn.models import EndOfStudy
from intecomm_reports.models import Diagnoses
from intecomm_screening.models import PatientLog


def get_diagnoses_df() -> pd.DataFrame:

    df = read_frame(Diagnoses.objects.all())
    df["site_id"] = df["site"].map({obj.domain: obj.id for obj in Site.objects.all()})
    df = df.drop(columns=["site"])
    df = df.reset_index(drop=True)

    # add group identifier
    qs_patientlog = PatientLog.objects.values("subject_identifier", "group_identifier").all()
    df_pat = read_frame(qs_patientlog)
    df = df.merge(df_pat, on="subject_identifier", how="left")

    # add demographics from consent
    df_consent = read_frame(
        SubjectConsent.objects.values("subject_identifier", "gender", "dob", "site").all()
    )
    df_consent["site_id"] = df_consent["site"].map(
        {obj.domain: obj.id for obj in Site.objects.all()}
    )
    df_consent = df_consent.drop(columns=["site"])
    df_consent = df_consent.reset_index(drop=True)

    df = df.merge(
        df_consent[["subject_identifier", "gender", "dob", "site_id"]],
        how="left",
        on=["subject_identifier", "site_id"],
    )

    df_eos = read_frame(
        EndOfStudy.objects.values(
            "subject_identifier",
            "offstudy_datetime",
            "offstudy_reason",
            "other_offstudy_reason",
            "site",
        ).all()
    )
    df_eos["site_id"] = df_eos["site"].map({obj.domain: obj.id for obj in Site.objects.all()})
    offstudy_reasons = {k: v for k, v in list_data.get("intecomm_lists.offstudyreasons")}

    df_eos["offstudy_reason"] = df_eos.apply(
        lambda row: (
            row["other_offstudy_reason"]
            if row["offstudy_reason"] == offstudy_reasons.get(OTHER)
            else row["offstudy_reason"]
        ),
        axis=1,
    )

    # recode out of catchment as transferred
    # ???

    df_eof = df_eos.drop(columns=["site"])
    df_eof = df_eof.reset_index(drop=True)
    df = df.merge(
        df_eof[
            [
                "subject_identifier",
                "offstudy_datetime",
                "offstudy_reason",
                "other_offstudy_reason",
                "site_id",
            ]
        ],
        how="left",
        on=["subject_identifier", "site_id"],
    )

    df["country"] = df["site_id"].apply(lambda x: sites.get(x).country.lower())

    return df



from .end_of_study import Eos

# from .on_study_missing_lab_values import OnStudyMissingLabValues
# from .on_study_missing_values import OnStudyMissingValues
from .subjects_transferred import SubjectsTransferred
from .vl import Vl



# Generated by Django 5.1 on 2024-10-04 12:59

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0020_auto_20240829_1604"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="OnStudyMissingLabValues",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("original_id", models.UUIDField(null=True)),
                ("label_lower", models.CharField(max_length=150, null=True)),
                ("subject_visit_id", models.UUIDField(null=True)),
                ("report_datetime", models.DateTimeField(null=True)),
                ("label", models.CharField(max_length=50, null=True)),
                ("visit_code", models.CharField(max_length=25, null=True)),
                ("visit_code_sequence", models.IntegerField(null=True)),
                ("schedule_name", models.CharField(max_length=25, null=True)),
                ("modified", models.DateTimeField(null=True)),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
            ],
            options={
                "verbose_name": "Missing Lab values for on-study patient",
                "verbose_name_plural": "Missing Lab values for on-study patients",
                "db_table": "onstudy_missing_lab_values_view",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
        migrations.CreateModel(
            name="OnStudyMissingValues",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("original_id", models.UUIDField(null=True)),
                ("label_lower", models.CharField(max_length=150, null=True)),
                ("subject_visit_id", models.UUIDField(null=True)),
                ("report_datetime", models.DateTimeField(null=True)),
                ("label", models.CharField(max_length=50, null=True)),
                ("visit_code", models.CharField(max_length=25, null=True)),
                ("visit_code_sequence", models.IntegerField(null=True)),
                ("schedule_name", models.CharField(max_length=25, null=True)),
                ("modified", models.DateTimeField(null=True)),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
            ],
            options={
                "verbose_name": "Missing values for on-study patient",
                "verbose_name_plural": "Missing values for on-study patients",
                "db_table": "onstudy_missing_values_view",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
        migrations.CreateModel(
            name="ApptLocationQa",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("created", models.DateTimeField()),
                (
                    "report_model",
                    models.CharField(default="meta_reports.appttype", max_length=50),
                ),
                ("visit_code", models.CharField(blank=True, max_length=50, null=True)),
                ("visit_code_sequence", models.IntegerField(default=0)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Appointment location QA",
                "verbose_name_plural": "Appointment location QA",
                "abstract": False,
                "default_permissions": ("view", "export", "viewallsites"),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"], name="intecomm_re_modifie_4638df_idx"
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="intecomm_re_user_mo_927bd2_idx",
                    ),
                ],
            },
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-27 00:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0017_auto_20240827_0323"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="eos",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "End of study",
                "verbose_name_plural": "End of study",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectstransferred",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "Subjects Transferred",
                "verbose_name_plural": "Subjects Transferred",
            },
        ),
        migrations.AlterModelOptions(
            name="vl",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "Viral loads",
                "verbose_name_plural": "Viral loads",
            },
        ),
    ]



# Generated by Django 5.1.2 on 2024-10-31 03:02

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0027_delete_onstudymissinglabvalues_and_more"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'intecomm_reports.vl' as `report_model` from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(MONTH, baseline_date, vl_date) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'intecomm_reports.vl_view' as `report_model` from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(MONTH, baseline_date, vl_date) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'intecomm_reports.vl' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'intecomm_reports.vl_view' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'intecomm_reports.vl' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'intecomm_reports.vl_view' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



from django.db import models
from django_pandas.managers import DataFrameManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions


class AppointmentLocationQa(
    UniqueSubjectIdentifierFieldMixin, QaReportModelMixin, BaseUuidModel
):
    """A read-only table of 1120 appointments where
    `appt_type`=`community`.
    """

    report_model = models.CharField(max_length=50, default="meta_reports.appttype")

    visit_code = models.CharField(max_length=50, null=True, blank=True)

    visit_code_sequence = models.IntegerField(default=0)

    objects = DataFrameManager()

    class Meta(UniqueSubjectIdentifierFieldMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Appointment location QA"
        verbose_name_plural = "Appointment location QA"
        default_permissions = qa_reports_permissions



# Generated by Django 5.1 on 2024-10-04 14:28

import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0022_rename_apptlocationqa_appointmentlocationqa_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="appointmentlocationqa",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow),
        ),
        migrations.AlterField(
            model_name="diagnoses",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow),
        ),
    ]



import pandas as pd

from ..constants import DM_HTN, DM_ONLY, HIV_DM, HIV_DM_HTN, HIV_HTN, HIV_ONLY, HTN_ONLY


def get_conditions_by_gender(df: pd.DataFrame, summarize: bool | None = None) -> pd.DataFrame:
    """Given a dataframe calculates a freq table for each condition
    by gender.

    For example:

        # with patient log
        patientlog_df = get_patientlog_df()
        df = get_conditions_by_gender(patientlog_df, summarize=True)

        # with diagnoses table
        dx_df = get_diagnoses_df()
        df = get_conditions_by_gender(dx_df, summarize=True)

    """

    s = df[(df["hiv"] == 1) & (df["htn"] == 0) & (df["dm"] == 0)]["gender"].value_counts()
    df1 = s.to_frame().reset_index()
    df1["label"] = HIV_ONLY
    cond_df = df1.copy()

    for hiv, htn, dm, label in [
        (0, 1, 0, HTN_ONLY),
        (0, 0, 1, DM_ONLY),
        (1, 0, 1, HIV_DM),
        (1, 1, 0, HIV_HTN),
        (1, 1, 1, HIV_DM_HTN),
        (0, 1, 1, DM_HTN),
    ]:
        s = df[(df["hiv"] == hiv) & (df["htn"] == htn) & (df["dm"] == dm)][
            "gender"
        ].value_counts()
        df1 = s.to_frame().reset_index()
        df1["label"] = label
        cond_df = pd.concat([cond_df, df1])
    cond_df = cond_df.reset_index(drop=True)
    if summarize:
        cond_df = cond_df.pivot(index="label", columns=["gender"], values=["count"])
        cond_df.columns = [col[1] for col in cond_df.columns]
        cond_df = cond_df.reset_index()
        cond_df["Total"] = cond_df["Female"] + cond_df["Male"]
        cond_df.loc[len(cond_df)] = {
            "label": "Total",
            "Female": cond_df["Female"].sum(),
            "Male": cond_df["Male"].sum(),
            "Total": cond_df["Total"].sum(),
        }
        cond_df["F_PROP"] = cond_df["Female"] / cond_df.loc[7]["Female"]
        cond_df["M_PROP"] = cond_df["Male"] / cond_df.loc[7]["Male"]
        cond_df["T_PROP"] = cond_df["Total"] / cond_df.loc[7]["Total"]
    return cond_df



import pandas as pd
from django.contrib.sites.models import Site
from django_pandas.io import read_frame

from intecomm_screening.models import PatientLog


def get_patientlog_df() -> pd.DataFrame:

    qs_patientlog = PatientLog.objects.values(
        *["patient_log_identifier", "conditions__name"]
    ).all()

    df_conditions = read_frame(qs_patientlog)
    df_conditions["hiv"] = df_conditions["conditions__name"].apply(
        lambda x: 1 if x == "HIV" else 0
    )
    df_conditions["htn"] = df_conditions["conditions__name"].apply(
        lambda x: 1 if x == "htn" else 0
    )
    df_conditions["dm"] = df_conditions["conditions__name"].apply(
        lambda x: 1 if x == "dm" else 0
    )
    df_conditions = df_conditions.pivot_table(
        index="patient_log_identifier", values=["hiv", "htn", "dm"], aggfunc="sum"
    )
    columns = [
        "site",
        "gender",
        "age_in_years",
        "patient_log_identifier",
        "screening_identifier",
        "subject_identifier",
        "group_identifier",
        "stable",
        "willing_to_screen",
        "screening_refusal_reason",
        "screening_refusal_reason_other",
    ]
    qs_patientlog = PatientLog.objects.values(*columns).all()
    df = read_frame(qs_patientlog)
    sites = {obj.name.title(): obj.id for obj in Site.objects.all()}
    df["site_id"] = df["site"].map(sites)

    df = df.merge(df_conditions, on="patient_log_identifier", how="left", suffixes=("", "_y"))

    # convert datetimes to date
    date_cols = list(df.select_dtypes(include=["datetime64", "datetime64[ns, UTC]"]).columns)
    for col in date_cols:
        df[col] = df[col].dt.normalize()

    return df



import pandas as pd
from django_pandas.io import read_frame
from edc_pdutils.dataframes import get_eos, get_subject_visit

from intecomm_prn.models import SubjectTransfer


def get_eos_df() -> pd.DataFrame:
    """
    df = get_eos_df()

    # look at transfers and last attended visit
    df[(df.transfer_reason.notna())]

    """
    df_eos = get_eos("intecomm_prn.endofstudy")
    df_visit = get_subject_visit("intecomm_subject.subjectvisit")
    df_last_visit = (
        df_visit.groupby("subject_identifier")
        .agg({"last_visit_code": "max", "last_visit_datetime": "max"})
        .reset_index()
    )

    df_eos = df_eos.merge(df_last_visit, on="subject_identifier", how="left")
    df_transfer = read_frame(
        SubjectTransfer.objects.values(
            "subject_identifier",
            "transfer_date",
            # "transfer_reason",
            "transfer_reason_other",
        ).all(),
        verbose=False,
    )
    df_transfer["transferred"] = 1
    df_reasons = read_frame(
        SubjectTransfer.objects.values("subject_identifier", "transfer_reason__name").all()
    )
    df_reasons["num"] = 1
    df_reasons = df_reasons.pivot_table(
        index="subject_identifier", columns="transfer_reason__name", values="num"
    ).reset_index()
    df_reasons = df_reasons.fillna(0)

    df_transfer = df_transfer.rename(columns={"transfer_reason": "transfer_reason_id"})
    df_transfer["transfer_date"] = df_transfer["transfer_date"].apply(pd.to_datetime)
    df_transfer["transfer_date"] = df_transfer["transfer_date"].dt.floor("d")
    df_transfer = df_transfer.merge(df_reasons, on="subject_identifier", how="left")
    df_transfer = df_transfer.reset_index(drop=True)

    df_eos = df_eos.merge(
        df_transfer,
        on="subject_identifier",
        how="left",
    )
    df_eos["transferred"] = df_eos["transferred"].fillna(0)

    return df_eos
